@@ -26,12 +26,14 @@
 
 void handle_hw_change(struct font_s *font, int old_height, int old_width, int old_length)
 {
-    status_msg("Applying changes to glyph size...");
+    //status_msg("Applying changes to glyph size...");
     
     char hskip = old_height/font->height;
     char rhskip = font->height/old_height;
     char wskip = old_width/font->width;
+    if(wskip && old_width%font->width) wskip++;
     char rwskip = font->width/old_width;
+    if(rwskip && font->width%old_width) rwskip++;
     int new_width = (font->width)/8;
     if(font->width%8) new_width++;
     int new_bytes = new_width;
@@ -48,6 +50,8 @@ void handle_hw_change(struct font_s *font, int old_height, int old_width, int ol
     char *newdata = (char *)malloc(new_datasize);
     if(!newdata) return;
     memset((void *)newdata, 0, new_datasize);
+
+    status_msg("Applying changes to glyph size...");
     
     int i = 0;
     int i2 = 0;
@@ -120,6 +124,8 @@ void handle_hw_change(struct font_s *font, int old_height, int old_width, int ol
 	  }
 	  j += old_bytes;
 	}
+	/* make sure we don't write past this char's size */
+	if(j2 >= new_charsize) break;
       }
       i += old_charsize;
       i2 += new_charsize;
@@ -538,10 +544,15 @@ skip:
 	  }
 	  else if(sel == 3)
 	  {
+            /* we deliberately avoid changing BDF font length for now.
+             * TODO: fix this!
+             */
+            if(font->version == VER_BDF) break;
 	    if(font->length < 256) font->length = 256;
 	    else if(font->length < 512) font->length = 512;
 	    else font->length = 256;
-	    struct module_s *mod = get_module_by_name(font->version < 3 ? "psf" : get_version_str(font->version));
+	    struct module_s *mod = get_module_by_name(font->version < 3 ? "psf" :
+                                                get_version_str(font->version));
 	    if(font->length > mod->max_length) font->length = mod->max_length;
 	    //if(font->version == VER_CP && font->length != 256) font->version = VER_PSF2;
 	    //if(font->version == VER_RAW && font->length != 256) font->version = VER_PSF2;
@@ -549,7 +560,8 @@ skip:
 	  else if(sel == 4)
 	  {
 	    font->has_unicode_table = !font->has_unicode_table;
-	    if(font->version == VER_CP || font->version == VER_RAW) font->has_unicode_table = 0;
+	    if(font->version == VER_CP || font->version == VER_RAW)
+                font->has_unicode_table = 0;
 	  }
 	  else if(sel == 5)
 	  {
@@ -565,6 +577,11 @@ skip:
 	    if(font->width > mod->max_width) font->width = mod->max_width;
 	    if(font->height > mod->max_height) font->height = mod->max_height;
 	    /* any width larger than 8 is incompatible with PSF1 & CP */
+            /* NOTE: we have to check width for PSF1 manually, as the
+             *       PSF module works for PSF2 limits which are bigger
+             *       than PSF1 limits.
+             */
+	    if(font->width > 8 && font->version == VER_PSF1) font->width = 8;
 	    /*
 	    if(font->version != VER_PSF2 && font->width > 8) font->width = 8;
 	    if(font->version == VER_CP || font->version == VER_RAW)
