@@ -274,7 +274,7 @@ struct font_s *bdf_load_font(unsigned char *file_data, long file_size)
 	if(!font) goto memory_error;
 	memset((void *)font, 0, sizeof(struct font_s));
 	unsigned char *data = (unsigned char *)NULL;
-	font->unicode_info_size = (256 * 4);
+	font->unicode_info_size = (512 * 4);
 	font->unicode_info = (unsigned char *)malloc(font->unicode_info_size);
 	if(!font->unicode_info) goto memory_error;
 	memset((void *)font->unicode_info, 0, font->unicode_info_size);
@@ -288,6 +288,8 @@ struct font_s *bdf_load_font(unsigned char *file_data, long file_size)
 	memcpy(font->metadata, (void *)&metadata_table, sizeof(metadata_table));
 	struct metadata_item_s *metadata = (struct metadata_item_s *)font->metadata;
 	font->version = get_version("BDF");
+        /* to be assigned later when we know char count */
+        struct char_info_s *char_info = NULL;
 
 	char *s = strtok((char *)file_data, "\n");
 	if(!s) goto file_read_error;
@@ -303,6 +305,7 @@ struct font_s *bdf_load_font(unsigned char *file_data, long file_size)
 	char scope = BDF_GLOBAL_SCOPE;
 	int w = 0, h = 0;
 	int globalw = 0, globalh = 0;
+        char globaln = 0;
 	int xoff = 0, yoff = 0;
 	int globalxoff = 0, globalyoff = 0;
 	int i, data_index = 0, chars = 0;
@@ -331,7 +334,12 @@ struct font_s *bdf_load_font(unsigned char *file_data, long file_size)
 		scope = BDF_GLOBAL_SCOPE;
 	  }
 	  else if(index == FONT_KEYWORD)
-		save_metadata_str(font, METADATA_FONT, get_string(&s));
+          {
+                skip_spaces(&s);
+                int z = 0;
+                while(*s != '\0') buf[z++] = *s++;
+		save_metadata_str(font, METADATA_FONT, buf); //get_string(&s));
+          }
 	  else if(index == COPYRIGHT_KEYWORD)
 		save_metadata_str(font, METADATA_COPYRIGHT, get_string(&s));
 	  else if(index == FONT_VERSION_KEYWORD)
@@ -389,6 +397,7 @@ struct font_s *bdf_load_font(unsigned char *file_data, long file_size)
 		globalyoff = get_integer(&s);
 		font->width = globalw;
 		font->height = globalh;
+                globaln = ((globalw+7)/8)*2;
 		font->charsize = font->height*((font->width+7)/8);
 		metadata[METADATA_FONTBOUNDINGBOX_X].value = globalw;
 		metadata[METADATA_FONTBOUNDINGBOX_Y].value = globalh;
@@ -410,6 +419,11 @@ struct font_s *bdf_load_font(unsigned char *file_data, long file_size)
 		memset((void *)data, 0, font->charsize*font->length);
 		font->data = data;
 		font->data_size = font->charsize*font->length;
+                font->unicode_info_size = (font->length * 4);
+                char_info = (struct char_info_s *)malloc(font->length*sizeof(struct char_info_s));
+                if(!char_info) goto memory_error;
+                font->char_info = (void *)char_info;
+                font->char_info_size = font->length*sizeof(struct char_info_s);
 	  }
 	  else if(index == STARTCHAR_KEYWORD)
 	  {
@@ -419,13 +433,15 @@ struct font_s *bdf_load_font(unsigned char *file_data, long file_size)
                 unsigned int chr = 0;
 		if(s[0] != 'U' || s[1] != '+')
 		{
-		  if(*s >= '2' && *s <= '7') chr = codepoint_to_unicode(s);
+		  //if(*s >= '2' && *s <= '7') chr = codepoint_to_unicode(s);
+		  if(*s >= '0' && *s <= '9') chr = get_hex(s, 4);
 		  else
                   {
                       chr = postscript_to_unicode(s);
                       if(!chr)
                       {
-                          chr = codepoint_to_unicode(s);
+                          //chr = codepoint_to_unicode(s);
+                          if(strlen(s) == 4) chr = get_hex(s, 4);
                       }
                   }
                   /*
@@ -481,7 +497,45 @@ struct font_s *bdf_load_font(unsigned char *file_data, long file_size)
 		h = get_integer(&s);
 		xoff = get_integer(&s);
 		yoff = get_integer(&s);
+                if(font->char_info)
+                {
+                    char_info[chars].BBw = w;
+                    char_info[chars].BBh = h;
+                    char_info[chars].BBXoff = xoff;
+                    char_info[chars].BBYoff = yoff;
+                }
 	  }
+	  else if(index == SWIDTH_KEYWORD)
+          {
+		if(scope != BDF_GLYPH_SCOPE) goto corrupt_file;
+                if(font->char_info)
+                {
+                    int swx1 = get_integer(&s);
+                    int swy1 = get_integer(&s);
+                    char_info[chars].swidthX = swx1;
+                    char_info[chars].swidthY = swy1;
+                }
+          }
+	  else if(index == DWIDTH_KEYWORD)
+          {
+		if(scope != BDF_GLYPH_SCOPE) goto corrupt_file;
+                if(font->char_info)
+                {
+                    int dwx1 = get_integer(&s);
+                    int dwy1 = get_integer(&s);
+                    char_info[chars].dwidthX = dwx1;
+                    char_info[chars].dwidthY = dwy1;
+                }
+          }
+	  else if(index == ENCODING_KEYWORD)
+          {
+		if(scope != BDF_GLYPH_SCOPE) goto corrupt_file;
+                if(font->char_info)
+                {
+                    int encoding = get_integer(&s);
+                    char_info[chars].encoding = encoding;
+                }
+          }
 	  else if(index == BITMAP_KEYWORD)
 	  {
 		if(scope != BDF_GLYPH_SCOPE) goto corrupt_file;
@@ -490,7 +544,7 @@ struct font_s *bdf_load_font(unsigned char *file_data, long file_size)
 		{
 		  if(!globalw || !globalh)
 			goto corrupt_file;
-		  w = globalw; h = globalh;
+		  //w = globalw; h = globalh;
 		}
 		i = h;
 		while(i && (s = strtok(NULL, "\n")))
@@ -509,6 +563,11 @@ struct font_s *bdf_load_font(unsigned char *file_data, long file_size)
 		    data[data_index+j] = line & 0xFF;
 		    line >>= 8;
 		  }
+		  n = globaln >> 1;
+		  for( ; j < n; j++)
+		  {
+		    data[data_index+j] = 0;
+                  }
 		  data_index += n;
 		  /*
 		  for(j = (1 << (w-1)); j >= 1; j >>= 1)
@@ -521,6 +580,20 @@ struct font_s *bdf_load_font(unsigned char *file_data, long file_size)
 		  i--;
 		}
 		if(i) goto corrupt_file;
+                /* fill the rest of lines (if any) with zeroes */
+                i = globalh-h;
+                while(i--)
+                {
+		  /* how many nibbles do we have per line? */
+		  char n = ((globalw+7)/8)*2;
+		  n >>= 1;
+		  unsigned int j;
+		  for(j = 0; j < n; j++)
+		  {
+		    data[data_index+j] = 0;
+		  }
+		  data_index += n;
+                }
 	  }
 	} while((s = strtok(NULL, "\n")));
 	/* check we got all chars */
@@ -542,7 +615,7 @@ struct font_s *bdf_load_font(unsigned char *file_data, long file_size)
 	font->module = get_module_by_name("bdf");
 	/* something REALLY WRONG happended here */
 	if(!font->module) goto undefined_error;
-	
+        
 	reset_all_cursors();
 	return font;
 
@@ -622,6 +695,7 @@ int bdf_write_to_file(FILE *file, struct font_s *font)
   res = fprintf(file, "CHARS %d\n", font->length);
   int i, j, l;
   unsigned char *data = font->data;
+  struct char_info_s *char_info = (struct char_info_s *)font->char_info;
   for(i = 0; i < font->length; i++)
   {
     res = fprintf(file, "STARTCHAR ");
@@ -656,9 +730,23 @@ int bdf_write_to_file(FILE *file, struct font_s *font)
     {
       fprintf(file, "U+%04X\n", i);
     }
-    res = fprintf(file, "ENCODING %d\n", i);
-    res = fprintf(file, "SWIDTH 1000 0\n");
-    res = fprintf(file, "DWIDTH %d 0\n", font->width);
+    
+    if(font->char_info)
+    {
+        res = fprintf(file, "ENCODING %d\n", char_info[i].encoding);
+        res = fprintf(file, "SWIDTH %d %d\n", char_info[i].swidthX, char_info[i].swidthY);
+        res = fprintf(file, "DWIDTH %d %d\n", char_info[i].dwidthX, char_info[i].dwidthY);
+        /*
+        res = fprintf(file, "BBX %d %d %d %d\n", char_info[i].BBw, char_info[i].BBh,
+					     char_info[i].BBXoff, char_info[i].BBYoff);
+        */
+    }
+    else
+    {
+        res = fprintf(file, "ENCODING %d\n", i);
+        res = fprintf(file, "SWIDTH 1000 0\n");
+        res = fprintf(file, "DWIDTH %d 0\n", font->width);
+    }
     res = fprintf(file, "BBX %d %d %d %d\n", meta[METADATA_FONTBOUNDINGBOX_X].value,
 					     meta[METADATA_FONTBOUNDINGBOX_Y].value,
 					     meta[METADATA_FONTBOUNDINGBOX_XOFF].value,
@@ -787,10 +875,12 @@ void bdf_handle_version_change(struct font_s *font, char old_version)
 	if(!font->metadata) return;
 	/* copy the standard metadata table, to fill it later */
 	memcpy(font->metadata, (void *)&metadata_table, sizeof(metadata_table));
-	struct metadata_item_s *metadata = (struct metadata_item_s *)font->metadata;
-	metadata[METADATA_FONTBOUNDINGBOX_X].value = font->width;
-	metadata[METADATA_FONTBOUNDINGBOX_Y].value = font->height;
   }
+  struct metadata_item_s *metadata = (struct metadata_item_s *)font->metadata;
+  metadata[METADATA_FONTBOUNDINGBOX_X].value = font->width;
+  metadata[METADATA_FONTBOUNDINGBOX_Y].value = font->height;
+  metadata[METADATA_FONTBOUNDINGBOX_XOFF].value = 0;
+  metadata[METADATA_FONTBOUNDINGBOX_YOFF].value = 0;
 }
 
 
