@@ -113,6 +113,14 @@ struct bdf_keyword_s
 #define ENDCHAR_KEYWORD			(44)
 
 
+struct char_info_s {
+    int encoding;
+    int swidthX, swidthY;
+    int dwidthX, dwidthY;
+    int BBw, BBh, BBXoff, BBYoff;
+};
+
+
 struct font_s *bdf_create_empty_font();
 struct font_s *bdf_load_font_file(char *file_name);
 struct font_s *bdf_load_font(unsigned char *file_data, long file_size);
