@@ -624,16 +624,6 @@ void export_unitab(struct font_s *font)
   fprintf(f, "# for sure that this is a fontopia unicode table file!.\n");
   fprintf(f, "#\n");
   
-  /*
-  if(font->version == VER_CP)
-  {
-    cp_export_unitab(font, f);
-    fclose(f);
-    free(fn);
-    return;
-  }
-  */
-  
   if(font->module->export_unitab)
   {
     font->module->export_unitab(font, f);
@@ -644,7 +634,7 @@ void export_unitab(struct font_s *font)
   
   int i;
   char t[8];
-  char m[81];
+  char m[(MAX_UNICODE_TABLE_ENTRIES*7)+1];
 
   for(i = 0; i < font->length; i++)
   {
