@@ -189,7 +189,13 @@ void refresh_right_window(struct font_s *font)
     }
     memset((void *)wc, 0, 5);
     make_utf8(wc, c);
-    if(wc[0] == 0xa0 || wc[0] == 0x0) putchar(' ');
+    //if(wc[0] == 0xa0 || wc[0] == 0x0) putchar('?');
+    if(wc[0] == 0xa0 || wc[0] < 0x20)
+    {
+        //putchar('?');
+        make_utf8(wc, 0xfffd);
+        printf("%s", wc);
+    }
     else printf("%s", wc);
     i++; col++;
     if(col == right_window.cols_per_row)
